﻿using System;
using System.Collections.Generic;

namespace SilverHtmlContent
{
	public class SilverHtmlAttributes : List<SilverHtmlAttributeItem>
	{

		public SilverHtmlAttributes()
		{
		}

		public SilverHtmlAttributeItem this[string attribute_name]
		{
			get
			{
				if( string.IsNullOrEmpty( attribute_name))
					return null;

				foreach (SilverHtmlAttributeItem item in this)
				{
					if( string.Compare( item.AttributeName, attribute_name, StringComparison.InvariantCultureIgnoreCase) == 0)
						return item;
				}
				return null;
			}
		}

		public SilverHtmlAttributeItem this[Html_style_common_attributes attribute_enum]
		{
			get
			{
				string		attribute_name	= SilverHtmlAttributeItem.HtmlStyleName(attribute_enum);

				return this[attribute_name];
			}
		}

		public SilverHtmlAttributeItem this[Html_common_properties attribute_enum]
		{
			get
			{
				string attribute_name = SilverHtmlAttributeItem.HtmlPropertyName(attribute_enum);

				return this[attribute_name];
			}
		}


		public new void Add(SilverHtmlAttributeItem item)
		{
			if( item == null || string.IsNullOrEmpty(item.AttributeName) || string.IsNullOrEmpty(item.AttributeValue))
				return;

			SilverHtmlAttributeItem	s	= this[item.AttributeName];

			if (s != null)
				s.AttributeValue = item.AttributeValue;
			else
			{
				base.Add( item);
			}
		}

		public void Add(string attribute_name, string attribute_value)
		{
			if ( string.IsNullOrEmpty(attribute_name) || string.IsNullOrEmpty(attribute_value))
				return;

			SilverHtmlAttributeItem s = this[attribute_name];

			if (s != null)
				s.AttributeValue = attribute_value;
			else
			{
				base.Add(new SilverHtmlAttributeItem( attribute_name, attribute_value));
			}
		}


		public void Add(Html_common_properties property_enum, string attribute_value)
		{
			string		attrib_name	= SilverHtmlAttributeItem.HtmlPropertyName(property_enum);
			
			this.Add( attrib_name, attribute_value);
		}


		public void Add(Html_style_common_attributes style_enum, string attribute_value)
		{
			string		attrib_name	= SilverHtmlAttributeItem.HtmlStyleName(style_enum);
			
			this.Add( attrib_name, attribute_value);
		}


		public string HtmlStyleString
		{
			get
			{
				string	str_ret	= "";

				foreach (SilverHtmlAttributeItem item in this)
				{
					if( str_ret.Length > 0)
						str_ret	+= " ";
					str_ret	+= item.HtmlStyleString;
				}
				return str_ret;
			}
		}

		public string HtmlPropertyString
		{
			get
			{
				string str_ret = "";

				foreach (SilverHtmlAttributeItem item in this)
				{
					if (str_ret.Length > 0)
						str_ret += " ";
					str_ret += item.HtmlPropertyString;
				}
				return str_ret;
			}
		}
	}
}
