﻿using System;
using System.Windows.Controls;
using System.Windows.Browser;

namespace SilverHtmlContent.controls
{
	public class SilverHtmlElement : UserControl
	{
		protected string				m_html_tag				= "div",
										m_html_id				= "";
		protected SilverHtmlAttributes	m_style					= new SilverHtmlAttributes(),
										m_properties			= new SilverHtmlAttributes();
		protected SilverHtmlElementList	m_children				= new SilverHtmlElementList();
		protected int					m_html_indent_level		= 0;

		protected HtmlElement			m_html_control			= null;

		public SilverHtmlElement()
		{
		}

		public SilverHtmlElement(string html_tag, string html_id, int indent_level)
		{
			HtmlTag				= html_tag;
			HtmlId				= html_id;
			HtmlIndent_level	= indent_level;
		}


		public SilverHtmlElement(Html_common_tags tag_enum, string html_id, int indent_level)
		{
			HtmlTag				= HtmlTagName(tag_enum);
			HtmlId				= html_id;
			HtmlIndent_level	= indent_level;

		}

		public HtmlElement Html_control
		{
			get { return m_html_control; }
		}

		public string HtmlTag
		{
			get { return m_html_tag; }
			set { m_html_tag = value; }
		}

		public string HtmlId
		{
			get { return m_html_id; }
			set { m_html_id = value; }
		}

		public int HtmlIndent_level
		{
			get { return m_html_indent_level; }
			set
			{
				if (value < 0)
					return;
				m_html_indent_level = value;
			}
		}

		public SilverHtmlAttributes HtmlStyle
		{
			get { return m_style; }
			set
			{
				if( value == null)
					return;
				m_style = value;
			}
		}

		public SilverHtmlAttributes HtmlProperties
		{
			get { return m_properties; }
			set
			{
				if (value == null)
					return;
				m_properties = value;
			}
		}

		public SilverHtmlElementList Children
		{
			get { return m_children; }
			set
			{
				if( value == null)
					return;

				m_children = value;
			}
		}


		public void AddStyle(Html_style_common_attributes style_attrib, string style_value)
		{
			if( m_style == null)
				m_style	= new SilverHtmlAttributes();

			m_style.Add( style_attrib, style_value);
		}


		public void SetHtmlStyle(string attribute_name, string str_value)
		{
			if(m_html_control == null || string.IsNullOrEmpty( attribute_name) || string.IsNullOrEmpty( str_value))
				return;
			
			m_html_control.SetStyleAttribute( attribute_name, str_value);
		}

		public void SetHtmlProperty(string property_name, string str_value)
		{
			if (m_html_control == null || string.IsNullOrEmpty(property_name) || string.IsNullOrEmpty(str_value))
				return;

			m_html_control.SetProperty(property_name, str_value);
		}


		public void AddProperty(Html_common_properties property_attrib, string style_value)
		{
			if (m_properties == null)
				m_properties = new SilverHtmlAttributes();

			m_properties.Add(property_attrib, style_value);
		}

		private HtmlElement CurrentHtmlElement(HtmlElement parent_html_element)
		{
			HtmlElement element = null;

			if (parent_html_element == null || parent_html_element.Children == null || parent_html_element.Children.Count <= 0)
			{
				element = HtmlPage.Document.CreateElement(m_html_tag);
				element.Id	= HtmlId;
			}
			else
			{
				foreach (HtmlElement elementx in parent_html_element.Children)
				{
					if (string.Compare(elementx.TagName, m_html_tag, StringComparison.InvariantCultureIgnoreCase) != 0)
						continue;

					if (string.Compare(elementx.Id, m_html_id, StringComparison.InvariantCultureIgnoreCase) == 0)
					{
						parent_html_element.RemoveChild( elementx);
						break;
					}
				}

			}

			if (element == null)
			{
				element = HtmlPage.Document.CreateElement(m_html_tag);
				element.Id = m_html_id;
			}
			parent_html_element.AppendChild(element);

			return element;
		}


		public void DisplayElement(HtmlElement parent_html_element)
		{
			if( parent_html_element == null)
				return;

			HtmlDocument	doc			= HtmlPage.Document;
			HtmlElement		element		= CurrentHtmlElement( parent_html_element);

			if( element == null)
				return;

			m_html_control	= element;

			element.Id		= HtmlId;
			
			foreach( SilverHtmlAttributeItem item in m_style)
				element.SetStyleAttribute( item.AttributeName, item.AttributeValue);

			foreach( SilverHtmlAttributeItem itemp in m_properties)
				element.SetAttribute( itemp.AttributeName, itemp.AttributeValue);

			if (m_children != null)
			{
				foreach (SilverHtmlElement child in m_children)
				{
					child.DisplayElement(element);
				}
			}
		}




		public new double Width
		{
			get
			{
				if (Html_control == null)
					return base.Width;

				double attrib = SilverHtmlContentCtrl.GetHtmlDimension(Html_control, "width", "offsetWidth", 0.0);

				return attrib;
			}
			set
			{
				if (Html_control != null)
					SilverHtmlContentCtrl.SetDimentionAttribute(Html_control, "width", value);
				base.Width = value;
			}
		}

		public new double Height
		{
			get
			{
				if (Html_control == null)
					return base.Height;

				double attrib = SilverHtmlContentCtrl.GetHtmlDimension(Html_control, "height", "offsetHeight", 0.0);

				return attrib;
			}
			set
			{
				if (Html_control != null)
					SilverHtmlContentCtrl.SetDimentionAttribute(Html_control, "height", (int)value);
				base.Height = value;
			}
		}

		#region *************** helpers ***********************


		protected string IndentString
		{
			get
			{
				if (m_html_indent_level <= 0)
					return "";
				string str_ret = "";

				return str_ret.PadLeft(m_html_indent_level, '\t');
			}
		}


		public string HtmlOutputString
		{
			get
			{
				string str_ret = "",
						style = m_style.HtmlStyleString,
						properties = m_properties.HtmlPropertyString;

				if (string.IsNullOrEmpty(m_html_tag))
					return str_ret;

				str_ret = IndentString + "<" + m_html_tag;

				/// add tag ID
				if (string.IsNullOrEmpty(m_html_id) == false)
					str_ret += " id=\"" + m_html_id + "\"";

				/// add tag style
				if (string.IsNullOrEmpty(style) == false)
					str_ret += " style=\"" + style + "\"";

				/// add tag properties
				if (string.IsNullOrEmpty(properties) == false)
					str_ret += " " + properties;

				/// append tag children if any, otherwise close tag
				if (m_children == null || m_children.Count <= 0)
					str_ret += "/>";
				else
				{
					str_ret += ">\r\n";
					str_ret += m_children.HtmlOutputString;	// add children
					str_ret += "</" + m_html_tag + ">";		// close tag
				}
				return str_ret;
			}
		}


		private static string Normalize_tag_enum_name(string enum_name)
		{
			if (string.IsNullOrEmpty(enum_name))
				return "";

			/// remove first underscore (ex: like for _class attribute name)
			if (enum_name.StartsWith("_"))
				enum_name = enum_name.Substring(1);

			/// replace underscores by hyphens
			return enum_name.Replace('_', '-');
		}

		public static string HtmlTagName(Html_common_tags attrib)
		{
			string str_ret = Enum.GetName(typeof(Html_common_tags), attrib);

			return Normalize_tag_enum_name(str_ret);
		}

		#endregion

	}
}
