﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Browser;
using System.ComponentModel;
using System.Windows.Media;


namespace SilverHtmlContent.controls
{
	public partial class SilverHtmlContentCtrl : UserControl
	{
#region ********** Dependency properties ********************************

		public static DependencyProperty LeftProperty = DependencyProperty.Register("Left", typeof(double),
														typeof(SilverHtmlContentCtrl),
														new PropertyMetadata(new PropertyChangedCallback(LeftPropertyChanged)));

		public static DependencyProperty TopProperty = DependencyProperty.Register("Top", typeof(double),
														typeof(SilverHtmlContentCtrl),
														new PropertyMetadata(new PropertyChangedCallback(TopPropertyChanged)));

		public static DependencyProperty HtmlWidthProperty = DependencyProperty.Register("HtmlWidth", typeof(string),
														typeof(SilverHtmlContentCtrl),
														new PropertyMetadata(new PropertyChangedCallback(HtmlWidthPropertyChanged)));

		public static DependencyProperty HtmlHeightProperty = DependencyProperty.Register("HtmlHeight", typeof(string),
														typeof(SilverHtmlContentCtrl),
														new PropertyMetadata(new PropertyChangedCallback(HtmlHeightPropertyChanged)));

		public static DependencyProperty TargetUrlProperty = DependencyProperty.Register("TargetUrl", typeof(string),
														typeof(SilverHtmlContentCtrl),
														new PropertyMetadata(new PropertyChangedCallback(TargetUrlPropertyChanged)));


		private static void LeftPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			if (obj == null)
				return;
			if (obj is SilverHtmlContentCtrl != true)
				return;

			SilverHtmlContentCtrl ctrl = (SilverHtmlContentCtrl)obj;

			ctrl.Left = (double)e.NewValue;
		}


		private static void TopPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			if (obj == null)
				return;
			if (obj is SilverHtmlContentCtrl != true)
				return;

			SilverHtmlContentCtrl ctrl = (SilverHtmlContentCtrl)obj;

			ctrl.Top = (double)e.NewValue;
		}

		private static void HtmlWidthPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			if (obj == null)
				return;
			if (obj is SilverHtmlContentCtrl != true)
				return;

			SilverHtmlContentCtrl ctrl = (SilverHtmlContentCtrl)obj;

			ctrl.HtmlWidth = (string)e.NewValue;
		}

		private static void HtmlHeightPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			if (obj == null)
				return;
			if (obj is SilverHtmlContentCtrl != true)
				return;

			SilverHtmlContentCtrl ctrl = (SilverHtmlContentCtrl)obj;

			ctrl.HtmlHeight = (string)e.NewValue;
		}

		private static void TargetUrlPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
		{
			if (obj == null)
				return;
			if (obj is SilverHtmlContentCtrl != true)
				return;

			SilverHtmlContentCtrl ctrl = (SilverHtmlContentCtrl)obj;

			ctrl.TargetUrl = (string)e.NewValue;
		}

#endregion

		protected SilverHtmlElement m_element		= new SilverHtmlElement( Html_common_tags.div, "silver_html_div", 0),
									m_i_frame		= new SilverHtmlElement( Html_common_tags.iframe, "silver_iframe", 1);

		protected double			m_left			= 0.0,
									m_top			= 0.0;
		protected string			m_html_width	= "",		// "100%",
									m_html_height	= "",		// "400px",
									m_source_url	= "";

		protected bool				m_initial_refresh		= true;
		protected bool				m_updating_dimensions	= false;

		public SilverHtmlContentCtrl()
		{
			InitializeComponent();
			init_instance();
		}


		private void init_instance()
		{
			m_element.SizeChanged += new SizeChangedEventHandler(m_element_SizeChanged);

			/// set the div position to absolute, set the z-order to display over our silverlight control
			m_element.AddStyle(Html_style_common_attributes.position, "absolute");
			//m_element.AddStyle(Html_style_common_attributes.z_order, "10");
			//Plugin_div.SetStyleAttribute("z-order", "0");

			m_element.Children.Add(m_i_frame);

			m_element.BorderThickness	= new Thickness(2.0);
			m_element.BorderBrush		= new SolidColorBrush(Colors.Black);
			LayoutRoot.Children.Clear();
			LayoutRoot.Children.Add(m_element);

			UpdateHtmlControlDimensions();	
		}

		void m_element_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			//Refresh();
		}


		private HtmlElement Plugin_div
		{
			get
			{
				return HtmlPage.Plugin.Parent;
			}
		}

		public double Top
		{
			get { return m_top; }
			set
			{
				m_top = value;
				SetValue(Canvas.TopProperty, value);

				SetDimentionAttribute(m_element.Html_control, "top", value);
				Initial_refresh();
			}
		}

		public double Left
		{
			get { return m_left; }
			set
			{
				m_left = value;
				SetValue(Canvas.TopProperty, value);
				SetDimentionAttribute(m_element.Html_control, "left", value);
				Initial_refresh();
			}
		}

		public bool Is_In_design_mode
		{
			get { return DesignerProperties.IsInDesignTool; }
		}

		public int Html_ZIndex
		{
			get { return (int) GetDimensionAttribute( m_element.Html_control, "z-index", 0); }
			set { SetDimentionAttribute(m_element.Html_control, "z-index", value); }
		}


		public new Visibility Visibility
		{
			get { return base.Visibility; }
			set
			{
				base.Visibility	= value;

				if( Is_In_design_mode || m_element == null)
					return;

				if (value == Visibility.Collapsed)
				{
					m_element.SetHtmlStyle("visibility", "hidden");
					//m_element.SetHtmlStyle("display", "none");
				}
				else
				{
					m_element.SetHtmlStyle("visibility", "visible");
					//m_element.SetHtmlStyle("display", "block");
				}
			}
		}

		public new double Width
		{
			get { return base.Width; }
			set
			{
				base.Width = value;
				UpdateHtmlControlDimensions();
			}
		}

		public new double Height
		{
			get { return base.Height; }
			set
			{
				base.Height = value;
				UpdateHtmlControlDimensions();
			}
		}


		public string HtmlWidth
		{
			get { return m_html_width; }
			set
			{
				if( value == null)
					return;

				m_html_width		= value;
				UpdateSilverControlDimensions();
			}
		}

		public string HtmlHeight
		{
			get { return m_html_height; }
			set
			{
				if (value == null)
					return;

				m_html_height = value;
				UpdateSilverControlDimensions();
			}
		}


		public string TargetUrl
		{
			get { return m_source_url; }
			set
			{
				m_source_url = value;
				m_i_frame.SetHtmlProperty( "src", m_source_url);	// .AddProperty(Html_common_properties.src, TargetUrl);
				Initial_refresh();
			}
		}

		
		private void UserControl_Loaded(object sender, RoutedEventArgs e)
		{
			if( Is_In_design_mode)
				return;

			Plugin_div.SetStyleAttribute("position", "relative");
			Initial_refresh();
		}


		private void RefreshXX()
		{
			if (Is_In_design_mode)
				return;

			
			////UpdateSilverControlDimensions();

			//m_element.AddStyle(Html_style_common_attributes.width, Width.ToString() + "px");	// HtmlWidth);
			//m_element.AddStyle(Html_style_common_attributes.height, Height.ToString() + "px");	// HtmlHeight);
			//m_element.AddStyle(Html_style_common_attributes.top, Top.ToString() + "px");
			//m_element.AddStyle(Html_style_common_attributes.left, Left.ToString() + "px");
			
			////m_i_frame.AddStyle(Html_style_common_attributes.width, "100%");
			////m_i_frame.AddStyle(Html_style_common_attributes.height, "100%");	// m_html_height);

			////m_i_frame.AddProperty(Html_common_properties.src, TargetUrl);
			//m_element.DisplayElement(Plugin_div);
		}


		private void Initial_refresh()
		{
			if (Is_In_design_mode || m_initial_refresh == false)
				return;

			//UpdateSilverControlDimensions();

			m_element.AddStyle(Html_style_common_attributes.width, Width.ToString() + "px");	// HtmlWidth);
			m_element.AddStyle(Html_style_common_attributes.height, Height.ToString() + "px");	// HtmlHeight);
			m_element.AddStyle(Html_style_common_attributes.top, Top.ToString() + "px");
			m_element.AddStyle(Html_style_common_attributes.left, Left.ToString() + "px");

			m_i_frame.AddStyle(Html_style_common_attributes.width, "100%");
			m_i_frame.AddStyle(Html_style_common_attributes.height, "100%");	// m_html_height);

			//m_i_frame.AddProperty(Html_common_properties.src, TargetUrl);
			m_element.DisplayElement(Plugin_div);

			m_initial_refresh	= false;
		}


		//private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
		//{
		//    Width		= e.NewSize.Width;
		//    Height		= e.NewSize.Height;
		//    //Refresh();
		//}


		protected void UpdateSilverControlDimensions()
		{
			if (m_updating_dimensions || Is_In_design_mode || m_element == null || m_element.Html_control == null)
				return;

			m_updating_dimensions	= true;
			double	Width	= GetHtmlDimension( m_element.Html_control, "width", "offsetWidth", 0.0),
					Height	= GetHtmlDimension( m_element.Html_control, "height", "offsetHeight", 0.0);
			m_updating_dimensions	= false;
		}

		protected void UpdateHtmlControlDimensions()
		{
			if (m_updating_dimensions || Is_In_design_mode || m_element == null || m_element.Html_control == null)
				return;

			m_updating_dimensions	= true;
			if( Width != double.NaN)
				SetDimentionAttribute(m_element.Html_control, "width", (int)Width);	// - Left - Margin.Left - Margin.Right);
			if( Height != double.NaN)
				SetDimentionAttribute(m_element.Html_control, "height", (int)Height);	// - Top - Margin.Top - Margin.Bottom);

			HtmlWidth	= ((int)Width).ToString()	+ "px";
			HtmlHeight	= ((int)Height).ToString()	+ "px";
			//Refresh();
			m_updating_dimensions	= false;
		}



		public static double GetDimensionProperty(HtmlElement html_ctrl, string property_name, double default_value)
		{
			if (html_ctrl == null)
			{
				//debug_display_div_error();
				return default_value;
			}

			string str_value = html_ctrl.GetProperty(property_name).ToString();

			if (string.IsNullOrEmpty(str_value))
			{
				return default_value;
			}
			double value = default_value;

			if (double.TryParse(str_value, out value) == false)
				return default_value;

			return value;
		}


		public static double GetDimensionAttribute(HtmlElement html_ctrl, string attrib_name, double default_value)
		{
			if (html_ctrl == null)
			{
				//debug_display_div_error();
				return default_value;
			}

			// try to get the style attribute value
			string str_value = html_ctrl.GetStyleAttribute(attrib_name);

			if (string.IsNullOrEmpty(str_value))
			{
				return default_value;
			}

			// remove the dimension's unit (px)
			str_value = str_value.Replace("px", "");

			double value = default_value;

			// try to convert to numeric value
			if (double.TryParse(str_value, out value) == false)
				return default_value;
			return value;
		}

		public static void SetDimentionAttribute(HtmlElement html_ctrl, string attrib_name, double value)
		{
			if (html_ctrl == null)
			{
				//debug_display_div_error();
				return;
			}

			html_ctrl.SetStyleAttribute(attrib_name, ((int)value).ToString() + "px");
		}

		public static double GetHtmlDimension(HtmlElement html_ctrl, string attrib_name, string property_name, double default_value)
		{
			double attrib = GetDimensionAttribute(html_ctrl, attrib_name, default_value);

			if (attrib <= 0.0)
				return GetDimensionProperty(html_ctrl, property_name, default_value);
			return attrib;
		}



		//protected override Size MeasureOverride(Size availableSize)
		//{
		//    if (!Is_In_design_mode)
		//    {
		//        Width = availableSize.Width;
		//        Height = availableSize.Height;
		//    }
		//    return base.MeasureOverride(availableSize);
		//}

		private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			if( Is_In_design_mode)
				return;

			Width	= e.NewSize.Width;
			Height	= e.NewSize.Height;
			//Refresh();
		}

	}
}
